@@ -1,8 +1,7 @@
 import time
 import string
 import logging
-from uuid import uuid4
-from collections import namedtuple
+from collections import namedtuple, OrderedDict
 from contextlib import contextmanager
 
 import xapian
@@ -12,10 +11,10 @@ from operator import itemgetter
 from functools import partial
 from json import loads
 
-from xapian import QueryParser, DocNotFoundError
+from xapian import Query, QueryParser, DocNotFoundError
 
 from . import snowball
-from .elements import Schema, String, Integer, Float, Array
+from .elements import Schema
 from .exc import ValidationError, PrefixError
 from .tools import LRUDict, lazy_property
 
@@ -63,54 +62,8 @@ def to_term(value, prefix=None):
     return _prefix(prefix) + value if prefix else value
 
 
-class QuerySchema(Schema):
-    type = String.using(default='xodbquery')
-    term = Array.of(String).using(prefix='query_term',
-                                  getter=lambda s, o, e: list(o))
-    id = Array.of(String).using(prefix='query_id',
-                                getter=lambda s, o, e: o.__xodb_id__)
-
-
-class Query(object):
-    __xodb_schema__ = QuerySchema
-
-    class __metaclass__(type):
-
-        def __getattr__(cls, name):
-            return getattr(xapian.Query, name)
-
-    def __init__(self, *args, **kwargs):
-        self.__xodb_query__ = kwargs.pop('query', None) or xapian.Query(*args, **kwargs)
-        self.__xodb_id__ = uuid4()
-
-    def __getattr__(self, name):
-        return getattr(self.__xodb_query__, name)
-
-    def __getstate__(self):
-        return dict(query=self.__xodb_query__.serialise(),
-                    id=str(self.__xodb_id__))
-
-    def __setstate__(self, state):
-        self.__xodb_query__ = xapian_Query.unserialise(state['query'])
-        self.__xodb_id__ = uuid.UUID(state['id'])
-
-    def __str__(self):
-        return str(self.__xodb_query__)
-
-    def __iter__(self):
-        return iter(self.__xodb_query__)
-
-
-class RecordSchema(Schema):
-    type = String.using(default='xodbrecord')
-    rank = Integer.using(getter=lambda s, o, e: o._xodb_rank)
-    percent = Integer.using(getter=lambda s, o, e: o._xodb_percent)
-    weight = Integer.using(getter=lambda s, o, e: o._xodb_weight)
-
-
 class Record(object):
     """Nice attribute-accessable record for a search result."""
-    __xodb_schema__ = RecordSchema
 
     def __init__(self, document, percent, rank, weight, query, db):
         self._xodb_document = document
@@ -833,7 +786,7 @@ class Database(object):
                 qp = self.get_query_parser(language, default_op, 
                                            retry_limit=retry_limit)
                 def query_op():
-                    return Query(query=qp.parse_query(query, parser_flags))
+                    return qp.parse_query(query, parser_flags)
                 result = self.retry_if_modified(query_op, retry_limit)
                 if not self.inmem:
                     self.query_cache[cache_key] = result
@@ -878,7 +831,7 @@ class Database(object):
                              default_op, parser_flags)
         if echo:
             print "Done parsing query: %s" % str(query)
-        enq.set_query(query.__xodb_query__)
+        enq.set_query(query)
 
         limit = limit or self.backend.get_doccount()
 
@@ -943,7 +896,7 @@ class Database(object):
                             yield record
                 # no errors exhuasting the set? break out and we're done
                 break
-            except xapian.DatabaseError, e:
+            except xapian.DatabaseError:
                 # an error occured, either, the db was closed, or the
                 # modified error happened two frequently in the inner
                 # loop, so we are going to replay the whole query
@@ -974,7 +927,7 @@ class Database(object):
         if echo:
             print str(query)
         enq = xapian.Enquire(self.backend)
-        enq.set_query(query.__xodb_query__)
+        enq.set_query(query)
 
         mset = self._build_mset(enq, retry_limit=retry_limit)
         return mset.size()
@@ -988,7 +941,8 @@ class Database(object):
               klimit=1.0,
               kmlimit=1.0,
               echo=False,
-              retry_limit=RETRY_LIMIT):
+              retry_limit=RETRY_LIMIT,
+              include_query_terms=True):
         """Get facet suggestions for the query, then the query with
         each suggested facet, asking xapian for an estimated count of
         each sub-query.
@@ -1010,7 +964,8 @@ class Database(object):
                                    kmlimit=kmlimit,
                                    echo=echo,
                                    retry_limit=retry_limit,
-                                   format_term=False)
+                                   format_term=False,
+                                   include_query_terms=include_query_terms)
         for facet in suggestions:
             q = Query(Query.OP_AND, [query, facet])
             if echo:
@@ -1030,7 +985,8 @@ class Database(object):
                translit=None,
                default_op=Query.OP_AND,
                parser_flags=default_parser_flags,
-               retry_limit=RETRY_LIMIT):
+               retry_limit=RETRY_LIMIT,
+               include_query_terms=True):
         """
         Expand a query on a given set of prefixes.
         """
@@ -1051,7 +1007,8 @@ class Database(object):
             q, prefix=head, language=language, echo=echo,
             limit=limit, mlimit=mlimit,
             klimit=klimit, kmlimit=kmlimit,
-            retry_limit=retry_limit).items()
+            retry_limit=retry_limit,
+            include_query_terms=include_query_terms).items()
 
         for name, score in r:
             if tail:
@@ -1065,7 +1022,7 @@ class Database(object):
                 results[(name, score)] = r
             else:
                 results[(name, score)] = score
-        return results
+        return OrderedDict(sorted(results.items(), key=lambda i: i[0][1], reverse=True))
 
     def estimate(self, query,
                  limit=0,
@@ -1091,7 +1048,7 @@ class Database(object):
                              default_op, parser_flags,
                              retry_limit=retry_limit)
 
-        enq.set_query(query.__xodb_query__)
+        enq.set_query(query)
         op = lambda: enq.get_mset(0, 0, limit)
         mset = self.retry_if_modified(op, retry_limit)
 
@@ -1150,7 +1107,8 @@ class Database(object):
                 parser_flags=default_parser_flags,
                 retry_limit=RETRY_LIMIT,
                 format_term=True,
-                collapse_stems=True):
+                collapse_stems=True,
+                include_query_terms=True):
         """
         Suggest terms that would possibly yield more relevant results
         for the given query.
@@ -1165,7 +1123,7 @@ class Database(object):
 
         if echo:
             print str(query)
-        enq.set_query(query.__xodb_query__)
+        enq.set_query(query)
 
         mset = self._build_mset(enq, offset=moffset, limit=mlimit,
                                 retry_limit=retry_limit)
@@ -1186,12 +1144,12 @@ class Database(object):
         if xapian_version <= (1, 2, 4):
             op = lambda: enq.get_eset(
                 limit, rset,
-                enq.INCLUDE_QUERY_TERMS,
+                enq.INCLUDE_QUERY_TERMS if include_query_terms else 0,
                 1.0, decider)
         else:
             op = lambda: enq.get_eset(
                 limit, rset,
-                enq.INCLUDE_QUERY_TERMS,
+                enq.INCLUDE_QUERY_TERMS if include_query_terms else 0,
                 1.0, decider, -3)
 
         stemmer = None
