@@ -28,10 +28,13 @@ def _prefix(prefix, value=None):
     return ("%s:%s" % (prefix, value)) if value else prefix
 
 
-def _normalize(value):
-    return unicodedata.normalize(
-        'NFKC', unicode(value).strip().lower()).encode('utf-8')
-
+def _normalize(value, lower=True):
+    if lower:
+        return unicodedata.normalize(
+            'NFKC', unicode(value).strip().lower()).encode('utf-8')
+    else:
+        return unicodedata.normalize(
+            'NFKC', unicode(value).strip()).encode('utf-8')
 
 _use_schema = object() # marker says use schema term generator
 
@@ -131,10 +134,6 @@ class Schema(SparseForm):
                     handler = self._handle_numericrange
                 elif isinstance(el, Location):
                     handler = self._handle_location
-                elif isinstance(el, Nilsimsa):
-                    handler = self._handle_nilsimsa
-                elif isinstance(el, Pickle):
-                    handler = self._handle_pickle
                 elif isinstance(el, List):
                     handler = self._handle_children
                 elif isinstance(el, Dict):
@@ -147,7 +146,7 @@ class Schema(SparseForm):
                 value = None
                 try:
                     value = handler(el, parent)
-                except InvalidTermError, e:
+                except InvalidTermError:
                     if self.ignore_invalid_terms:
                         logger.warning('Invalid term ignored: %r' % el)
                     else:
@@ -168,11 +167,11 @@ class Schema(SparseForm):
         name = element.flattened_name()
         memo = self._memo
         if term:
-            if element.lower:
-                term = term.lower()
             if element.prefix:
-                memo.add_term(_prefix(name, term), element.boolean, element.wdf_inc)
+                prefixed = _normalize(_prefix(name, term), lower=element.lower)
+                memo.add_term(prefixed, element.boolean, element.wdf_inc)
             else:
+                term = _normalize(term, lower=element.lower)
                 memo.add_term(term, element.boolean, element.wdf_inc)
             if element.sortable:
                 memo.add_value(name, value, type)
@@ -282,22 +281,6 @@ class Schema(SparseForm):
             memo.add_value(element.name, h, 'location')
         return True
 
-    def _handle_nilsimsa(self, element, parent):
-        memo = self._memo
-        try:
-            val = (parent[element.from_field].value
-                   .encode('translit/long')
-                   .encode('ascii', 'ignore'))
-            element.u = nilsimsa.Nilsimsa([val]).hexdigest()
-        except Exception:
-            print 'whoops: ', val
-            return
-        if element.u:
-            return self._handle_scalar(element.u, element.u, element, 'string')
-        return True
-
-    def _handle_pickle(self, element, parent):
-        return True
 
 class _BaseElement(object):
 
@@ -529,15 +512,6 @@ class NumericRange(_BaseRange):
     """
 
 
-class Nilsimsa(schema.String, _BaseElement):
-
-    from_field = None
-    """What other field the simhash should be computed from.
-
-    If this is not provided, indexing will raise an error.
-    """
-
-
 class Location(schema.Compound, _BaseElement):
     """ Compound location is a 2-tuple of lat/lon coordinates.
     """
@@ -571,22 +545,3 @@ class Location(schema.Compound, _BaseElement):
         return geoprint.encode(lat, lon, radians=radians)
 
 
-class Pickle(schema.Scalar, _BaseElement):
-    # broken
-
-    def serialize(self, value):
-        return unicode(cPickle.dumps(value).encode('base64'))
-
-    def adapt(self, value):
-        if value is None:
-            return None
-        if isinstance(value, basestring):
-            try:
-                return cPickle.loads(str(value).decode('base64'))
-            except cPickle.UnpicklingError:
-                raise AdaptationError()
-        else:
-            raise AdaptationError()
-
-    def getter(self, sc, ob, el):
-        return unicode(cPickle.dumps(ob).encode('base64'))
