@@ -45,6 +45,7 @@ decodemap = dict((k, i) for (i, k) in enumerate(alphabet))
 
 EARTH_RADIUS = 6378100
 
+
 def encode(latitude, longitude, precision=22, radians=False, box=False):
     """Encode the given latitude and longitude into a geoprint that
     contains 'precision' characters.
@@ -97,7 +98,7 @@ def encode(latitude, longitude, precision=22, radians=False, box=False):
     return result
 
 
-def decode(geoprint, radians=False):
+def decode(geoprint, radians=False, box=False):
     """Decode a geoprint, returning the latitude and longitude.  These
     coordinates should approximate the input coordinates within a
     degree of error returned by 'error()'
@@ -142,7 +143,11 @@ def decode(geoprint, radians=False):
     lat = (lati.min + lati.max) / 2
     lon = (loni.min + loni.max) / 2
     if radians:
-        return rads(lat), rads(lon)
+        lati = interval(rads(lati.min), rads(lati.max))
+        loni = interval(rads(loni.min), rads(loni.max))
+        lat, lon = rads(lat), rads(lon)
+    if box:
+        return (geoprint, (lati[0], loni[0]), (lati[1], loni[1]))
     return lat, lon
 
 
@@ -231,27 +236,23 @@ _radials = {'N' : 0,              # N
 
 
 def neighbors(geoprint, bearing=True, box=False):
-    """
-    Return the eight neighboring geoprints and optionally their
-    compass bearing flag around the given geoprint.
-
-    If bearing is True, bearing flag will be either None (not
-    adjacent) or one of ['N', 'NE', 'E', 'SE', 'S', 'SW', 'W', 'NW']
-    """
     results = set()
     precision = len(geoprint)
-    lat1, lon1 = decode(geoprint, radians=True)
-    d = size(geoprint) / EARTH_RADIUS
-    for b, r in _radials.iteritems():
-        lat = asin(sin(lat1) * cos(d) +
-                   cos(lat1) * sin(d) * cos(r))
-        dlon = atan2(sin(r) * sin(d) * cos(lat1),
-                     cos(d) - sin(lat1) * sin(lat))
-        lon = mod(lon1 + dlon + pi, 2 * pi) - pi
-        h = encode(lat, lon, precision=precision, radians=True, box=box)
-        if bearing:
-            h = (b, h)
-        results.add(h)
+    spacing = 180 / (2.0 ** (precision - 1))
+    ctr = decode(geoprint)
+    dirs = ['N', 'S', 'E', 'NE', 'SE', 'W', 'NW', 'SW']
+    i = 0
+    for direction_lat in (0, 1, -1):
+        for direction_long in (0, 1, -1):
+            if direction_lat == 0 and direction_long == 0:
+                continue
+            lat = ctr[0] + (direction_lat * spacing)
+            lon = ctr[1] + (direction_long * spacing)
+            h = encode(lat, lon, precision=precision, box=box)
+            if bearing:
+                h = (dirs[i], h)
+            results.add(h)
+            i += 1
     return results
 
 
@@ -260,11 +261,19 @@ def adjacent(first, second):
     Return a flag indicating if two geoprints are adjacent to each
     other.
     """
+    if first == second:
+        return False
     if min(len(first), len(second)) < 3:
         raise TypeError(
             "Adjacency requires at least 3 characters of precision.")
-    maxe = (error(first, True) / 2) + (error(second, True) / 2)
-    maxe2 = pow(maxe, 2)
-    h = sqrt(maxe2 + maxe2)
-    d = distance(first, second, True)
-    return h <= d
+    if len(first) != len(second):
+        raise TypeError('Adjacency can only be checked for the same precision')
+    lat1, lng1 = decode(first)
+    lat2, lng2 = decode(second)
+    precision = len(first)
+    spacing = 180 / (2.0 ** (precision - 1))
+
+    return ((abs(lat1 - lat2) == spacing and (lng1 == lng2)) or
+            (abs(lng1 - lng2) == spacing and (lat1 == lat2)) or
+            (abs(lat1 - lat2) == spacing and
+             abs(lng1 - lng2) == spacing))
